@@ -3,6 +3,7 @@ use strict;
 use warnings;
 use parent qw( Plack::Component );
 use RDF::LinkedData;
+use URI::NamespaceMap;
 use Plack::Request;
 
 =head1 NAME
@@ -193,8 +194,9 @@ the distribution for example.
 
 By adding an C<add_void> config key, you can make pass a file to the
 generator so that arbitrary RDF can be added to the VoID
-description. This is a very preliminary implementation, but it solves
-many use cases as much of the VoID description cannot be simply
+description. It will check the last modification time of the file and
+only update the VoID description if it has been modified. This is
+usefil since as much of the VoID description cannot be simply
 generated. To use it, the configuration would in JSON look something
 like this:
 
@@ -203,7 +205,10 @@ like this:
 where C<file> is the full path to RDF that should be added and
 C<syntax> is needed by the parser to parse it.
 
-
+Normally, the VoID description is cached in RAM and the store ETag is
+checked on every request to see if the description must be
+regenerated. If you use the C<add_void> feature, you can force
+regeneration on the next request by touching the file.
 
 =head1 FEEDBACK WANTED
 
@@ -255,18 +260,23 @@ sub prepare_app {
 															 void_config => $config->{void},
 															 base_uri => $config->{base_uri}
 															);
-	$self->{linkeddata}->namespaces($config->{namespaces}) if ($config->{namespaces});
+	$self->{linkeddata}->namespaces(URI::NamespaceMap->new($config->{namespaces})) if ($config->{namespaces});
 }
 
 sub call {
 	my($self, $env) = @_;
 	my $req = Plack::Request->new($env);
+	my $uri = $req->uri;
 	my $ld = $self->{linkeddata};
-	unless (($req->method eq 'GET') || ($req->method eq 'HEAD')) {
+	my $endpoint_path;
+	if ($ld->has_endpoint) {
+	  $endpoint_path = $ld->endpoint_config->{endpoint_path};
+	}
+	unless (($req->method eq 'GET') || ($req->method eq 'HEAD')
+			  || (($req->method eq 'POST') && defined($endpoint_path) && ($uri =~ m|$endpoint_path$|))) {
 		return [ 405, [ 'Content-type', 'text/plain' ], [ 'Method not allowed' ] ];
 	}
 
-	my $uri = $req->uri;
 	if ($uri->as_iri =~ m!^(.+?)/?(page|data)$!) {
 		$uri = URI->new($1);
 		$ld->type($2);
@@ -285,7 +295,7 @@ Kjetil Kjernsmo, C<< <kjetilk@cpan.org> >>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2010-2012 Kjetil Kjernsmo
+Copyright 2010-2013 Kjetil Kjernsmo
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
