@@ -1,29 +1,37 @@
 package RDF::LinkedData;
 
+use Moose;
 use namespace::autoclean;
 
 use RDF::Trine qw[iri literal blank statement];
 use RDF::Trine::Serializer;
+use RDF::Trine::Namespace;
 use Log::Log4perl qw(:easy);
 use Plack::Response;
 use RDF::Helper::Properties;
+use URI::NamespaceMap;
 use URI;
 use HTTP::Headers;
 use Module::Load::Conditional qw[can_load];
-use Moose;
 use MooseX::UndefTolerant::Attribute;
 use Encode;
 use RDF::RDFa::Generator 0.102;
 use HTML::HTML5::Writer qw(DOCTYPE_XHTML_RDFA);
 use Data::Dumper;
+use Digest::MD5 ('md5_hex');
+
 
 with 'MooseX::Log::Log4perl::Easy';
 
 BEGIN {
 	if ($ENV{TEST_VERBOSE}) {
-		Log::Log4perl->easy_init( { level   => $TRACE } );
+		Log::Log4perl->easy_init( { level   => $TRACE,
+											 category => 'RDF.LinkedData' 
+										  } );
 	} else {
-		Log::Log4perl->easy_init( { level   => $FATAL } );
+		Log::Log4perl->easy_init( { level   => $FATAL,
+											 category => 'RDF.LinkedData' 
+										  } );
 	}
 }
 
@@ -32,15 +40,15 @@ BEGIN {
 
 =head1 NAME
 
-RDF::LinkedData - A simple Linked Data implementation
+RDF::LinkedData - A simple Linked Data server implementation
 
 =head1 VERSION
 
-Version 0.56
+Version 0.60
 
 =cut
 
-our $VERSION = '0.56';
+ our $VERSION = '0.60';
 
 
 =head1 SYNOPSIS
@@ -99,6 +107,8 @@ Called by Moose to initialize an object.
 
 =cut
 
+# TODO look at buildargs to replace UndefTolerant
+
 sub BUILD {
 	my $self = shift;
 
@@ -113,6 +123,11 @@ sub BUILD {
 		unless (can_load( modules => { 'RDF::Endpoint' => 0.03 })) {
 			throw Error -text => "RDF::Endpoint not installed. Please install or remove its configuration.";
 		}
+
+		unless (defined($self->endpoint_config->{endpoint_path})) {
+		  $self->endpoint_config->{endpoint_path} = '/sparql';
+		}
+
 		$self->endpoint(RDF::Endpoint->new($self->model, $self->endpoint_config));
  	} else {
 		$self->logger->info('No endpoint config found');
@@ -136,7 +151,7 @@ sub BUILD {
 	}
 }
 
-has store => (is => 'rw', isa => 'HashRef' );
+has store => (is => 'rw', isa => 'HashRef | Str' );
 
 
 =item C<< model >>
@@ -150,15 +165,22 @@ has model => (is => 'ro', isa => 'RDF::Trine::Model', lazy => 1, builder => '_bu
 
 sub _build_model {
 	my $self = shift;
+	return $self->_load_model($self->store);
+}
+
+sub _load_model {
+	my ($self, $store_config) = @_;
 	# First, set the base if none is configured
 	my $i = 0;
-	foreach my $source (@{$self->store->{sources}}) {
-		unless ($source->{base_uri}) {
-			${$self->store->{sources}}[$i]->{base_uri} = $self->base_uri;
+	if (ref($store_config) eq 'HASH') {
+		foreach my $source (@{$store_config->{sources}}) {
+			unless ($source->{base_uri}) {
+				${$store_config->{sources}}[$i]->{base_uri} = $self->base_uri;
+			}
+			$i++;
 		}
-		$i++;
 	}
-	my $store = RDF::Trine::Store->new( $self->store );
+	my $store = RDF::Trine::Store->new( $store_config );
 	return RDF::Trine::Model->new( $store );
 }
 
@@ -182,6 +204,7 @@ has void_config => (is => 'rw', traits => [ qw(MooseX::UndefTolerant::Attribute)
 								isa=>'HashRef', predicate => 'has_void_config');
 
 
+
 =item C<< request ( [ $request ] ) >>
 
 Returns the L<Plack::Request> object if it exists or sets it if a L<Plack::Request> object is given as parameter.
@@ -210,9 +233,33 @@ Gets or sets the namespaces that some serializers use for pretty-printing.
 
 =cut
 
-has 'namespaces' => (is => 'rw', isa => 'HashRef', default => sub { { rdf => 'http://www.w3.org/1999/02/22-rdf-syntax-ns#' } } );
+has 'namespaces' => (is => 'rw', 
+							isa => 'URI::NamespaceMap',
+							builder => '_build_namespaces',
+							lazy => 1,
+							handles => {
+											'add_namespace_mapping' => 'add_mapping',
+											'list_namespaces' => 'list_namespaces'
+										  });
 
 
+sub _build_namespaces {
+  my ($self, $ns_hash) = @_;
+  return $ns_hash || URI::NamespaceMap->new({ rdf => 'http://www.w3.org/1999/02/22-rdf-syntax-ns#' });
+}
+
+# Just a temporary compatibility hack
+sub _namespace_hashref {
+  my $self = shift;
+  my %hash;
+  foreach my $prefix ($self->namespaces->list_prefixes) {
+	 $hash{$prefix} = $self->namespaces->namespace_uri($prefix)->as_string;
+  }
+  return \%hash;
+}
+
+  
+
 
 =item C<< response ( $uri ) >>
 
@@ -227,13 +274,13 @@ sub response {
 	my $response = Plack::Response->new;
 
 	my $headers_in = $self->request->headers;
-	my $endpoint_path = '/sparql';
-	if ($self->has_endpoint_config && defined($self->endpoint_config->{endpoint_path})) {
-      $endpoint_path = $self->endpoint_config->{endpoint_path};
-	}
 
-	if ($self->has_endpoint && ($uri->path eq $endpoint_path)) {
-      return $self->endpoint->run( $self->request );
+	my $endpoint_path;
+	if ($self->has_endpoint) {
+	  $endpoint_path = $self->endpoint_config->{endpoint_path};
+	  if ($uri->path eq $endpoint_path) {
+		 return $self->endpoint->run( $self->request );
+	  }
 	}
 
 	if ($self->has_void) {
@@ -260,7 +307,7 @@ sub response {
 			if ($headers_in->can('header') && $headers_in->header('Accept')) {
 				$self->logger->debug('Found Accept header: ' . $headers_in->header('Accept'));
 			} else {
-				$headers_in->header(HTTP::Headers->new('Accept' => 'application/rdf+xml'));
+				$headers_in->header('Accept' => 'application/rdf+xml');
 				if ($headers_in->header('Accept')) {
 					$self->logger->warn('Setting Accept header: ' . $headers_in->header('Accept'));
 				} else {
@@ -271,7 +318,9 @@ sub response {
 			my $content = $self->_content($node, $type, $endpoint_path);
 			$response->headers->header('Vary' => join(", ", qw(Accept)));
 			if (defined($self->current_etag)) {
-				$response->headers->header('ETag' => $self->current_etag);
+				$self->current_etag =~ m|(^W/)|; # If the ETag is declared as weak, preserve that
+				my $weak = defined($1) ? $1 : '';
+				$response->headers->header('ETag' => $weak . md5_hex($self->current_etag . $content->{content_type}));
 			}
 			$response->headers->content_type($content->{content_type});
 			$response->body(encode_utf8($content->{body}));
@@ -358,6 +407,7 @@ sub count {
 	return $self->model->count_statements( $node, undef, undef );
 }
 
+
 # =item C<< _content ( $node, $type, $endpoint_path) >>
 #
 # Private method to return the a hashref with content for this URI,
@@ -371,6 +421,7 @@ sub count {
 
 sub _content {
 	my ($self, $node, $type, $endpoint_path) = @_;
+	
 	my $model = $self->model;
 	my $iter = $model->bounded_description($node);
 	my %output;
@@ -378,17 +429,18 @@ sub _content {
 		$self->{_type} = 'data';
 		my ($ctype, $s) = RDF::Trine::Serializer->negotiate('request_headers' => $self->request->headers,
 																			base => $self->base_uri,
-																			namespaces => $self->namespaces);
+																			namespaces => $self->_namespace_hashref);
 		$output{content_type} = $ctype;
 		if ($self->hypermedia) {
+			my $data_iri = iri($node->uri_value . '/data');
 			my $hmmodel = RDF::Trine::Model->temporary_model;
 			if($self->has_void) {
-				$hmmodel->add_statement(statement(iri($node->uri_value . '/data'), 
+				$hmmodel->add_statement(statement($data_iri, 
 															 iri('http://rdfs.org/ns/void#inDataset'), 
 															 $self->void->dataset_uri));
 			} else {
 				if($self->has_endpoint) {
-					$hmmodel->add_statement(statement(iri($node->uri_value . '/data'), 
+					$hmmodel->add_statement(statement($data_iri, 
 																 iri('http://rdfs.org/ns/void#inDataset'), 
 																 blank('void')));
 					$hmmodel->add_statement(statement(blank('void'), 
@@ -396,13 +448,13 @@ sub _content {
 																 iri($self->base_uri . $endpoint_path)));
 				}
 				if($self->namespaces_as_vocabularies) {
-					$hmmodel->add_statement(statement(iri($node->uri_value . '/data'), 
+					$hmmodel->add_statement(statement($data_iri, 
 																 iri('http://rdfs.org/ns/void#inDataset'), 
 																 blank('void')));
-					foreach my $nsuri (values(%{$self->namespaces})) {
+					foreach my $nsuri ($self->list_namespaces) {
 						$hmmodel->add_statement(statement(blank('void'), 
 																	 iri('http://rdfs.org/ns/void#vocabulary'),
-																	 iri($nsuri)));
+																	 iri($nsuri->uri)));
 					}
 				}
 			}
@@ -421,7 +473,7 @@ sub _content {
 		my $gen  = RDF::RDFa::Generator->new( style => 'HTML::Pretty',
 														  title => $preds->title( $node ),
 														  base => $self->base_uri,
-														  namespaces => $self->namespaces);
+														  namespaces => $self->_namespace_hashref);
 		my $writer = HTML::HTML5::Writer->new( charset => 'ascii', markup => 'html' );
 		$output{body} = $writer->document($gen->create_document($returnmodel));
 		$output{content_type} = 'text/html';
@@ -464,7 +516,7 @@ sub _negotiate {
 	eval {
 		($ct, $s) = RDF::Trine::Serializer->negotiate('request_headers' => $headers_in,
 																	 base_uri => $self->base_uri,
-																	 namespaces => $self->namespaces,
+																	 namespaces => $self->_namespace_hashref,
 																	 extend => {
 																					'text/html' => 'html',
 																					'application/xhtml+xml' => 'xhtml'
@@ -522,7 +574,7 @@ sub _void_content {
 				$generator->urispace($self->base_uri);
 			}
 			if ($self->namespaces_as_vocabularies) {
-				$generator->add_vocabularies(values(%{$self->namespaces}));
+				$generator->add_vocabularies($self->list_namespaces);
 			}
 			if ($self->has_endpoint) {
 				$generator->add_endpoints($self->base_uri . $endpoint_path);
@@ -556,7 +608,7 @@ sub _void_content {
 			my $gen = RDF::RDFa::Generator->new( style => 'HTML::Pretty',
 															 title => $self->void_config->{pagetitle} || 'VoID Description',
 															 base => $self->base_uri,
-															 namespaces => $self->namespaces);
+															 namespaces => $self->_namespace_hashref);
 			my $markup = ($ct eq 'application/xhtml+xml') ? 'xhtml' : 'html';
 			my $writer = HTML::HTML5::Writer->new( charset => 'ascii', markup => $markup );
 			$body = $writer->document($gen->create_document($self->_voidmodel));
@@ -568,7 +620,9 @@ sub _void_content {
 		$etag = $self->_last_extvoid_mtime if ($self->void_config->{add_void});
 		$etag .= $self->last_etag if (defined($self->last_etag));
 		if ($etag) {
-			$response->headers->header('ETag' => $etag);
+			$etag =~ m|(^W/)|; # If the ETag is declared as weak, preserve that
+			my $weak = defined($1) ? $1 : '';
+			$response->headers->header('ETag' => $weak . md5_hex($etag . $ct));
 		}
 		$response->headers->content_type($ct);
 		$response->body(encode_utf8($body));
@@ -619,6 +673,8 @@ L<http://lists.perlrdf.org/listinfo/dev>
 
 =item * Use a environment variable for config on the command line?
 
+=item * Make the result graph configurable.
+
 =back
 
 
@@ -630,7 +686,11 @@ almost totally rewritten.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2010 Gregory Todd Williams and ABC Startsiden AS, 2010-2012 Kjetil Kjernsmo
+Copyright 2010 Gregory Todd Williams
+
+Copyright 2010 ABC Startsiden AS
+
+Copyright 2010, 2011, 2012, 2013 Kjetil Kjernsmo
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
